package com.redxun.knowledge.album.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.StringUtils;
import com.redxun.knowledge.album.entity.dao.*;
import com.redxun.knowledge.album.entity.dto.AlbumApprovalRemarkDto;
import com.redxun.knowledge.album.entity.enums.AlbumVersionApproveEnum;
import com.redxun.knowledge.album.entity.enums.AlbumVersionEnum;
import com.redxun.knowledge.album.entity.vo.AlbumApproveFlowVo;
import com.redxun.knowledge.album.entity.vo.AlbumApproveListVo;
import com.redxun.knowledge.album.entity.vo.AlbumApproveVo;
import com.redxun.knowledge.album.entity.vo.AlbumYelpApproveListVo;
import com.redxun.knowledge.album.mapper.*;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.KnowledgeApprove;
import com.redxun.knowledge.mapper.KnowledgeApproveMapper;
import com.redxun.knowledge.utils.PageListUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * [业务--专辑审批表]业务服务类
 */
@Service
public class AlbumApproveServiceImpl extends SuperServiceImpl<AlbumApproveMapper, AlbumApprove> implements BaseService<AlbumApprove> {

    @Resource
    private AlbumApproveMapper albumApproveMapper;

    @Autowired
    private AlbumVersionMapper albumVersionMapper;

    @Autowired
    private AlbumInfoMapper albumInfoMapper;

    @Autowired
    private AlbumApprovalNodeMapper albumApprovalNodeMapper;

    @Autowired
    private AlbumCategoryMapper albumCategoryMapper;

    @Autowired
    private UserService userService;

    @Autowired
    private AlbumApproveServiceImpl albumApproveService;

    @Autowired
    private KnowledgeApproveMapper knowledgeApproveMapper;

    @Override
    public BaseDao<AlbumApprove> getRepository() {
        return albumApproveMapper;
    }

    /**
     * PC端根据知识专辑ID获取审批流程列表数据
     *
     * @param albumId
     * @param versionId
     * @return
     */
    public AlbumApproveFlowVo approvals(String albumId, String versionId) {
        //判断登录人是否是专辑所属人(是 查看审核中的版本(没有审核中的,查看已发布的) 不是 查看已发布的版本)
        String userId = userService.queryLoginUser();
        AlbumInfo albumInfo = albumInfoMapper.selectById(albumId);
        //返回结果
        AlbumApproveFlowVo albumApproveFlowVo = new AlbumApproveFlowVo();
        if (albumInfo.getOwnerId().equals(userId)) {
            //查询版本信息(审核中)
            AlbumVersion albumVersion1 = albumVersionMapper.
                    selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", albumId).eq("STATE", 1));
            //查询版本信息(驳回)
            AlbumVersion albumVersion4 = albumVersionMapper.
                    selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", albumId).eq("STATE", 4));
            if (albumVersion1 != null) {
                albumApproveFlowVo.setState(albumVersion1.getState());
                albumApproveFlowVo.setStateName(AlbumVersionEnum.getMessage(albumVersion1.getState()));
                List<AlbumApproveVo> approvals = getApprovals(albumVersion1.getPkId());
                albumApproveFlowVo.setFlows(approvals);
                return albumApproveFlowVo;
            } else if (albumVersion4 != null) {
                albumApproveFlowVo.setState(albumVersion4.getState());
                albumApproveFlowVo.setStateName(AlbumVersionEnum.getMessage(albumVersion4.getState()));
                List<AlbumApproveVo> approvals = getApprovals(albumVersion4.getPkId());
                albumApproveFlowVo.setFlows(approvals);
                return albumApproveFlowVo;
            } else {
                //生效中的专辑
                AlbumVersion albumVersion2 = albumVersionMapper.
                        selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", albumId).eq("STATE", 2));
                albumApproveFlowVo.setState(albumVersion2.getState());
                albumApproveFlowVo.setStateName(AlbumVersionEnum.getMessage(albumVersion2.getState()));
                List<AlbumApproveVo> approvals = getApprovals(albumVersion2.getPkId());
                albumApproveFlowVo.setFlows(approvals);
                return albumApproveFlowVo;
            }
        } else {
            //生效中的专辑
            AlbumVersion albumVersion2 = albumVersionMapper.
                    selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", albumId).eq("STATE", 2));
            if (albumVersion2 != null) {
                albumApproveFlowVo.setState(albumVersion2.getState());
                albumApproveFlowVo.setStateName(AlbumVersionEnum.getMessage(albumVersion2.getState()));
                List<AlbumApproveVo> approvals = getApprovals(albumVersion2.getPkId());
                albumApproveFlowVo.setFlows(approvals);
            }
            return albumApproveFlowVo;
        }
    }

    /**
     * PC获取审核流程数据
     *
     * @param versionId
     * @return
     */
    private List<AlbumApproveVo> getApprovals(String versionId) {
        List<AlbumApprove> albumApproveList = albumApproveMapper.selectList(new LambdaQueryWrapper<AlbumApprove>().
                eq(AlbumApprove::getVersionId, versionId).orderByAsc(AlbumApprove::getSort));
        //结果集
        List<AlbumApproveVo> albumApproveVoList = new ArrayList<>();
        boolean flag = true;
        for (int i = 0; i < albumApproveList.size(); i++) {
            AlbumApproveVo albumApproveVo = new AlbumApproveVo();
            BeanUtils.copyProperties(albumApproveList.get(i), albumApproveVo);
            albumApproveVo.setComment(albumApproveList.get(i).getRemark());
            if (albumApproveList.get(i).getResult() == null) {
                if (flag) {
                    albumApproveVo.setApprovalStatus("提交审核");
                    flag = false;
                } else {
                    albumApproveVo.setApprovalStatus("");
                }
            } else if (albumApproveList.get(i).getResult() == 0) {
                albumApproveVo.setApprovalStatus("审核驳回");
                flag = false;
            } else if (albumApproveList.get(i).getResult() == 1) {
                albumApproveVo.setApprovalStatus("审核通过");
            }
            albumApproveVoList.add(albumApproveVo);
        }
        return albumApproveVoList;
    }

    /**
     * 管理端审核列表
     *
     * @param queryFilter
     * @return
     */
    @Override
    public IPage query(QueryFilter queryFilter) {
        Map<String, Object> params = PageHelper.constructParams(queryFilter);
        String type = (String) queryFilter.getParams().get("type");
        //获取全部数据
        List<AlbumApprove> albumApproveList = albumApproveMapper.findAllAlbumApprove(queryFilter.getParams(), params);
        //根据专辑Id进行数据分组
        Map<String, List<AlbumApprove>> albumApproveMap = albumApproveList.stream().collect(Collectors.groupingBy(AlbumApprove::getAlbumId));
        // 返回结果集
        List<AlbumApprove> albumApproveVos = new ArrayList<>();
        // 遍历每组的数据
        Set<Map.Entry<String, List<AlbumApprove>>> entries = albumApproveMap.entrySet();
        for (Map.Entry<String, List<AlbumApprove>> entry : entries) {
            List<AlbumApprove> value = entry.getValue();
            for (int i = 0; i < value.size(); i++) {
                // 如果是最终审核通过,直接放入记录
                if (value.get(i).getAlbumVersion().getApprovalState() == 5) {
                    albumApproveVos.add(value.get(i));
                    break;
                }
                // 判断审批结果是否为null(如果审批结果没有null，证明审批已结束;如果等于0，证明审批没通过)
                if (value.get(i).getResult() == null || value.get(i).getResult() == 0) {
                    // 把该记录加到结果集中
                    albumApproveVos.add(value.get(i));
                    break;
                }
            }
        }
        //判断是否查看审核列表和知识管理员 是 查询全部数据 不是 只查看自己能审核的数据
        //查询知识管理员
        List<String> knowledgeManager = knowledgeApproveMapper.findKnowledgeManager();
        String user = userService.queryLoginUser();
        if ("1".equals(type) && knowledgeManager.contains(user)) {
            List<AlbumApprove> result = albumApproveVos.stream().
                    sorted((t1, t2) -> t2.getCreateTime().compareTo(t1.getCreateTime())).collect(Collectors.toList());
            Page pageResult = PageListUtils.getPages(queryFilter.getPage().getCurrent(), queryFilter.getPage().getSize(), result);
            List<AlbumApprove> records = pageResult.getRecords();
            List<AlbumApproveListVo> collect = records.stream().map(this::pageInfo).collect(Collectors.toList());
            pageResult.setRecords(collect);
            return pageResult;
        } else {
            //获取当前登录人的userId和deptId
            String userId = userService.queryLoginUser();
            String deptId = userService.queryLoginUserDeptId();

            List<AlbumApprove> albumApproveListVo1 = new ArrayList<>();
            //根据部门Id获取子部门信息
            //过滤数据
            albumApproveVos.forEach(albumApproveListVo -> {
                Integer count = albumApprovalNodeMapper.selectAllByDeptId(deptId, albumApproveListVo.getApproverId());
                List<String> collect = Arrays.stream(albumApproveListVo.getApproverId().split(",")).collect(Collectors.toList());
                if ((collect.contains(userId) || count > 0) &&
                        (albumApproveListVo.getAlbumVersion().getApprovalState() == 1 || albumApproveListVo.getAlbumVersion().getApprovalState() == 2)) {
                    albumApproveListVo1.add(albumApproveListVo);
                }
            });
            List<AlbumApprove> result = albumApproveListVo1.stream().
                    sorted((t1, t2) -> t2.getCreateTime().compareTo(t1.getCreateTime())).collect(Collectors.toList());
            Page pageResult = PageListUtils.getPages(queryFilter.getPage().getCurrent(), queryFilter.getPage().getSize(), result);
            List<AlbumApprove> records = pageResult.getRecords();
            List<AlbumApproveListVo> collect = records.stream().map(this::pageInfo).collect(Collectors.toList());
            pageResult.setRecords(collect);
            return pageResult;
        }
    }


    /**
     * 返回值赋值
     *
     * @param albumApprove
     * @return
     */
    private AlbumApproveListVo pageInfo(AlbumApprove albumApprove) {
        AlbumApproveListVo albumApproveListVo = new AlbumApproveListVo();
        albumApproveListVo.setPkId(albumApprove.getAlbumInfo().getPkId());
        albumApproveListVo.setAlbumName(albumApprove.getAlbumInfo().getName());
        String categoryId = albumApprove.getAlbumInfo().getCategoryId();
        AlbumCategory albumCategory = albumCategoryMapper.selectById(categoryId);
        if (albumCategory.getGrade() == 1) {
            albumApproveListVo.setCategoryName(albumCategory.getName());
        } else {
            AlbumCategory albumCategoryParent = albumCategoryMapper.
                    selectOne(new LambdaQueryWrapper<AlbumCategory>().eq(AlbumCategory::getPkId, albumCategory.getParent()));
            albumApproveListVo.setCategoryName(albumCategoryParent.getName() + "|" + albumCategory.getName());
        }
        albumApproveListVo.setOwnerName(userService.queryOsUserDto(albumApprove.getAlbumInfo().getOwnerId()).getFullName());
        albumApproveListVo.setCreateTime(albumApprove.getAlbumVersion().getCreateTime());
        albumApproveListVo.setApprovalStateName(AlbumVersionApproveEnum.getMessage(albumApprove.getAlbumVersion().getApprovalState()));
        albumApproveListVo.setApproverId(albumApprove.getApproverId());
        albumApproveListVo.setVersionId(albumApprove.getAlbumVersion().getPkId());
        albumApproveListVo.setApprovalId(albumApprove.getPkId());
        albumApproveListVo.setApprovalState(albumApprove.getAlbumVersion().getApprovalState());
        Integer knowledgeTotal = albumInfoMapper.selectCountKnowledge(albumApprove.getAlbumInfo().getPkId(), albumApprove.getAlbumVersion().getPkId());
        albumApproveListVo.setKnowledgeTotal(knowledgeTotal);
        //动作标记
        String userId = userService.queryLoginUser();
        String deptId = userService.queryLoginUserDeptId();
        Integer count = albumApprovalNodeMapper.selectAllByDeptId(deptId, albumApproveListVo.getApproverId());
        List<String> actionFlag = new ArrayList<>();
        List<String> collect = Arrays.stream(albumApproveListVo.getApproverId().split(",")).collect(Collectors.toList());
        if ((collect.contains(userId) || count > 0) &&
                (albumApprove.getAlbumVersion().getApprovalState() == 1 || albumApprove.getAlbumVersion().getApprovalState() == 2)) {
            actionFlag.add("approve");
        } else {
            actionFlag.add("view");
        }
        albumApproveListVo.setActionFlag(actionFlag);
        // 如果是最终审核通过(赋值实际审核人和审核时间)
        if (albumApprove.getAlbumVersion().getApprovalState() == 5 || albumApprove.getAlbumVersion().getApprovalState() == 4
                || albumApprove.getAlbumVersion().getApprovalState() == 3) {
            String actualApproverName = albumApprove.getActualApproverName();
            albumApproveListVo.setPreviousApproverName(actualApproverName);
            Date approvalTime = albumApprove.getApprovalTime();
            albumApproveListVo.setPreviousApproverTime(approvalTime);
        } else {
            //取前一条记录实际审核人和审核时间
            QueryWrapper<AlbumApprove> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("VERSION_ID", albumApprove.getVersionId());
            queryWrapper.eq("ALBUM_ID", albumApprove.getAlbumId());
            queryWrapper.eq("SORT", albumApprove.getSort() - 1);
            queryWrapper.orderByAsc("SORT");
            AlbumApprove albumApproveBySort = albumApproveMapper.selectOne(queryWrapper);
            if (albumApproveBySort != null) {
                String actualApproverName = albumApproveBySort.getActualApproverName();
                albumApproveListVo.setPreviousApproverName(actualApproverName);
                Date approvalTime = albumApproveBySort.getApprovalTime();
                albumApproveListVo.setPreviousApproverTime(approvalTime);
            }
        }
        return albumApproveListVo;

    }

    /**
     * 获取当前登陆人待审核的专辑数量
     *
     * @return
     */
    public Long total() {
        //构建查询条件
        QueryFilter queryFilter = new QueryFilter();
        Map<String, Object> params = new HashMap<>();
        params.put("type", "2");
        queryFilter.setParams(params);
        IPage page = albumApproveService.query(queryFilter);
        long total = page.getTotal();
        return total;
    }

    /**
     * 对知识专辑审核中的版本进行审核(快速审核和批量审核)
     *
     * @param albumApprovalRemarkDto
     */
    @Transactional(rollbackFor = Exception.class)
    public Integer approve(AlbumApprovalRemarkDto albumApprovalRemarkDto) {
        List<String> approvalIdList = albumApprovalRemarkDto.getPkIds();
        // 如果审批结论为null,置位空字符串
        if (StringUtils.isEmpty(albumApprovalRemarkDto.getRemark())) {
            albumApprovalRemarkDto.setRemark("");
        }
        //判断是否审核过该记录且是否有权限
        for (String approvalId : approvalIdList) {
            AlbumApprove albumApprove = albumApproveMapper.selectById(approvalId);
            if (albumApprove.getResult() != null) {
                //已经审核过
                return 1;
            }
            //获取当前登录人的userId和deptId
            String userId = userService.queryLoginUser();
            String deptId = userService.queryLoginUserDeptId();
            Integer count = albumApprovalNodeMapper.selectAllByDeptId(deptId, albumApprove.getApproverId());
            List<String> collect = Arrays.stream(albumApprove.getApproverId().split(",")).collect(Collectors.toList());
            if (!collect.contains(userId)) {
                //没有权限
                if (count <= 0) {
                    return 2;
                } else {
                    break;
                }

            }
        }
        approvalIdList.forEach(approvalId -> {
            //插入审核结论
            AlbumApprove albumApprove = new AlbumApprove();
            albumApprove.setPkId(approvalId);
            albumApprove.setResult(albumApprovalRemarkDto.getResult());
            albumApprove.setRemark(albumApprovalRemarkDto.getRemark());
            albumApprove.setActualApprover(userService.queryLoginUser());
            albumApprove.setActualApproverName(userService.queryLoginFullName());
            albumApprove.setApprovalTime(new Date());
            albumApproveMapper.updateById(albumApprove);
            //修改版本状态
            AlbumApprove albumApproveById = albumApproveMapper.selectById(approvalId);
            AlbumApprove albumApproveByIdNext = albumApproveMapper.
                    selectOne(new QueryWrapper<AlbumApprove>().
                            eq("SORT", albumApproveById.getSort() + 1).eq("VERSION_ID", albumApproveById.getVersionId()));
            //如果审核节点不是终审节点且专辑审核未通过,修改专辑版本状态为节点驳回
            if (albumApproveById.getIsFinal() == 0 && albumApprovalRemarkDto.getResult() == 0) {
                AlbumVersion albumVersion = new AlbumVersion();
                albumVersion.setPkId(albumApproveById.getVersionId());
                //专辑驳回
                albumVersion.setState(4);
                //专辑审核驳回
                albumVersion.setApprovalState(3);
                albumVersionMapper.updateById(albumVersion);
            }
            //如果审核节点是终审节点且专辑审核未通过,修改专辑版本状态为最终驳回
            if (albumApproveById.getIsFinal() == 1 && albumApprovalRemarkDto.getResult() == 0) {
                AlbumVersion albumVersion = new AlbumVersion();
                albumVersion.setPkId(albumApproveById.getVersionId());
                //专辑驳回
                albumVersion.setState(4);
                //专辑审核驳回
                albumVersion.setApprovalState(4);
                albumVersionMapper.updateById(albumVersion);
            }
            //如果审核节点是终审节点且专辑审核通过,修改专辑版本状态为最终通过(即审核完成)
            if ((albumApproveById.getIsFinal() == 1 || albumApproveByIdNext == null) && albumApprovalRemarkDto.getResult() == 1) {
                AlbumVersion albumVersion = new AlbumVersion();
                //更改其他专辑为未生效
                albumVersion.setState(3);
                albumVersionMapper.update(albumVersion, new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", albumApproveById.getAlbumId()));
                //专辑生效(其他专辑未生效)
                albumVersion.setPkId(albumApproveById.getVersionId());
                albumVersion.setState(2);
                //专辑审核最终通过
                albumVersion.setApprovalState(5);
                albumVersionMapper.updateById(albumVersion);
            }
            // 如果当前节点的下个节点为终审节点通过且当前节点审核通过,修改状态为待最终审核
            if (albumApproveByIdNext != null && albumApproveByIdNext.getIsFinal() == 1 && albumApprovalRemarkDto.getResult() == 1) {
                AlbumVersion albumVersion = new AlbumVersion();
                albumVersion.setPkId(albumApproveById.getVersionId());
                albumVersion.setApprovalState(2);
                albumVersionMapper.updateById(albumVersion);
            }
        });
        return -1;
    }

    /**
     * 根据版本Id删除记录
     *
     * @param pkId
     */
    public void deleteByVersionId(String pkId) {
        albumApproveMapper.deleteByVersionId(pkId);
    }
}
