
package com.redxun.knowledge.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.annotation.ClassDefine;
import com.redxun.common.base.db.BaseService;

import com.redxun.common.base.entity.JsonPageResult;
import com.redxun.common.base.entity.JsonResult;
import com.redxun.common.base.entity.QueryData;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.base.search.QueryFilterBuilder;
import com.redxun.common.tool.StringUtils;
import com.redxun.common.utils.ExceptionUtil;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.KnowledgeApprovalNode;
import com.redxun.knowledge.entity.dao.KnowledgeApprovalNodeFlower;
import com.redxun.knowledge.entity.dao.KnowledgeCategory;
import com.redxun.knowledge.entity.dto.KnowledgeApprovalNodeDto;
import com.redxun.knowledge.entity.dto.KnowledgeApprovalNodeFlowerDto;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalNodeFlowerVo;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalNodeVo;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalVo;
import com.redxun.knowledge.mapper.KnowledgeCategoryMapper;
import com.redxun.knowledge.service.KnowledgeApprovalNodeServiceImpl;
import com.redxun.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@RestController
@RequestMapping("/api-knowledge/knowledge/knowledgeApprovalNode")
@Api(tags = "业务--知识审批节点表")
@ClassDefine(title = "业务--知识审批节点表", alias = "KnowledgeApprovalNodeController", path = "/knowledge/knowledgeApprovalNode", packages = "", packageName = "子系统名称")
public class KnowledgeApprovalNodeController extends BaseController<KnowledgeApprovalNode> {

    @Autowired
    KnowledgeApprovalNodeServiceImpl knowledgeApprovalNodeService;

    @Autowired
    private KnowledgeCategoryMapper knowledgeCategoryMapper;

    @Autowired
    private UserService userService;

    @Override
    public BaseService getBaseService() {
        return knowledgeApprovalNodeService;
    }

    @Override
    public String getComment() {
        return "业务--知识审批节点";
    }

    @ApiOperation("查询审核流程节点列表")
    @GetMapping("queryApprovalNodes")
    public JsonResult queryApprovalNodes() {
        JsonResult jsonResult = JsonResult.getSuccessResult("获取数据成功");
        List<KnowledgeApprovalVo> approvalNodes = knowledgeApprovalNodeService.queryApprovalNodes();
        jsonResult.setData(approvalNodes);
        return jsonResult;
    }

    @ApiOperation("批量添加审核流程")
    @PostMapping("batchInsertApprovalNodes")
    public JsonResult batchInsertApprovalNodes(@RequestBody List<KnowledgeApprovalNodeDto> knowledgeApprovalNodeDto) {
        JsonResult jsonResult = JsonResult.getSuccessResult("添加数据成功");
        // 清除原有数据
        knowledgeApprovalNodeService.deleteKnowledgeApprovalNode();
        // 批量添加数据
        List<KnowledgeApprovalNode> knowledgeApprovalNodeList = knowledgeApprovalNodeDto
                .parallelStream()
                .map(e -> {
                    KnowledgeApprovalNode knowledgeApprovalNode = new KnowledgeApprovalNode();
                    BeanUtils.copyProperties(e, knowledgeApprovalNode);
                    return knowledgeApprovalNode;
                }).collect(Collectors.toList());
        knowledgeApprovalNodeService.saveBatch(knowledgeApprovalNodeList);
        jsonResult.setData(knowledgeApprovalNodeList);
        return jsonResult;
    }

    @ApiOperation(value = "分页查询审核流程节点")
    @ApiImplicitParam(name = "queryData", value = "查询条件")
    @PostMapping("findAllApprovalNodes")
    public JsonPageResult findAllApprovalNodes(@RequestBody QueryData queryData) {
        JsonPageResult jsonResult = JsonPageResult.getSuccess("返回数据成功!");
        try {
            QueryFilter filter = QueryFilterBuilder.createQueryFilter(queryData);
            IPage page = knowledgeApprovalNodeService.findAllApprovalNodes(filter);
            this.handlePage(page);
            jsonResult.setPageData(page);
        } catch (Exception e) {
            jsonResult.setSuccess(false);
            this.logger.error(ExceptionUtil.getExceptionMessage(e));
            jsonResult.setMessage(ExceptionUtil.getExceptionMessage(e));
        }
        return jsonResult;
    }

    @ApiOperation("添加审核流程(新)")
    @PostMapping("insertApprovalNodes")
    public JsonResult insertApprovalNodes(@Valid @RequestBody KnowledgeApprovalNodeFlowerDto knowledgeApprovalNodeFlowerDto, BindingResult validResult) throws Exception{
        JsonResult jsonResult = JsonResult.getSuccessResult("添加审核流程成功！");
        if (validResult.hasErrors()) {
            return jsonResult.setMessage("流程名称不能为空");
        }

        //返回到前端
        Map<String,Object> result = new HashMap<>();
        if (!CollectionUtils.isEmpty(knowledgeApprovalNodeFlowerDto.getCategoryId())){
            List<String> categoryName = new ArrayList<>();
            knowledgeApprovalNodeFlowerDto.getCategoryId().forEach(h -> {
                KnowledgeCategory knowledgeCategory = knowledgeCategoryMapper.selectById(h);
                if (!Objects.isNull(knowledgeCategory)){
                    categoryName.add(knowledgeCategory.getName());
                }
            });
            result.put("category",knowledgeApprovalNodeFlowerDto.getCategoryId());
            result.put("categoryName",categoryName);

        }

        try {
            String pkId = knowledgeApprovalNodeService.insertApprovalNodes(knowledgeApprovalNodeFlowerDto);
            result.put("pkId",pkId);
            jsonResult.setData(result);
        } catch (Exception e) {
            jsonResult.setData(result);
            jsonResult.setMessage(e.getMessage());
            jsonResult.setSuccess(false);
        }
        return jsonResult;
    }

    @ApiOperation("查询审核流程")
    @GetMapping("findAllNodes")
    public JsonResult findAllNodes() {
        List<KnowledgeApprovalNodeFlowerVo> knowledgeApprovalNodeFlowerVoList = knowledgeApprovalNodeService.findAllNodes();
        return JsonResult.getSuccessResult(knowledgeApprovalNodeFlowerVoList);
    }

    @ApiOperation("根据分类查询审核流程")
    @GetMapping("findAllNodesByCategoryId")
    public JsonResult findAllNodesByCategoryId(@RequestParam(value = "categoryId",required = false) String categoryId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        try {
            List<KnowledgeApprovalNodeVo> result = knowledgeApprovalNodeService.findAllNodesByCategoryId(categoryId,userService.queryLoginUser());
            jsonResult.setData(result);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setSuccess(false);
        }

        return jsonResult;
    }

    @ApiOperation("删除审核流程")
    @GetMapping("delApprovalNode")
    public JsonResult delApprovalNode(@RequestParam("pkId") String pkId){
        knowledgeApprovalNodeService.delApprovalNode(pkId);
        return JsonResult.getSuccessResult("删除成功");
    }

    @ApiOperation("查询选过的关联分类id集")
    @GetMapping("findAllCategorySelected")
    public JsonResult findAllCategorySelected(@RequestParam(value = "pkId",required = false) String pkId){
        List<String> idList =  knowledgeApprovalNodeService.findAllCategorySelected(pkId);
        return JsonResult.getSuccessResult(idList);
    }
}

