package com.redxun.knowledge.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.model.JPaasUser;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.IdGenerator;
import com.redxun.common.tool.StringUtils;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.KnowledgeAgent;
import com.redxun.knowledge.entity.dao.KnowledgeApprove;
import com.redxun.knowledge.entity.dto.KnowledgeAgentDto;
import com.redxun.knowledge.entity.vo.KnowledgeAgentVo;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalListVo;
import com.redxun.knowledge.mapper.KnowledgeAgentMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.stream.Collectors;

/**
 * 文件名: KnowledgeAgentServiceImpl
 * 作者: zizg
 * 时间: 2023/6/5
 * 描述:
 * 修改人:
 * 修改时间:
 * 修改内容:
 */
@Service
public class KnowledgeAgentServiceImpl extends SuperServiceImpl<KnowledgeAgentMapper, KnowledgeAgent> implements BaseService<KnowledgeAgent> {

    @Autowired
    private UserService userService;

    @Autowired
    private KnowledgeAgentMapper knowledgeAgentMapper;

    @Override
    public BaseDao<KnowledgeAgent> getRepository() {
        return knowledgeAgentMapper;
    }

    /**
     * 新增代理人
     * @param knowledgeAgentDto
     * @return
     */
    public String create(KnowledgeAgentDto knowledgeAgentDto) {
        if (knowledgeAgentDto.getTagerNo().equals(userService.queryLoginUsername())){
            throw new RuntimeException("代理人不能为自己！");
        }
        KnowledgeAgent knowledgeAgent = new KnowledgeAgent();
        BeanUtils.copyProperties(knowledgeAgentDto,knowledgeAgent);
        knowledgeAgent.setUserNo(userService.queryLoginUsername());
        if (StringUtils.isEmpty(knowledgeAgent.getPkId())){
            //查询当前登陆人是否已经被代理了
            KnowledgeAgent knowledgeAgentOne = knowledgeAgentMapper.
                    selectOne(new LambdaQueryWrapper<KnowledgeAgent>().eq(KnowledgeAgent::getUserNo,knowledgeAgent.getUserNo()));
            if (knowledgeAgentOne != null){
                throw new RuntimeException("当前用户已经被他人代理,添加失败！");
            }
            knowledgeAgent.setPkId(IdGenerator.getIdStr());
            knowledgeAgentMapper.insert(knowledgeAgent);
        } else {
            knowledgeAgentMapper.updateById(knowledgeAgent);
        }
        return knowledgeAgent.getPkId();
    }

    /**
     * 查询代理人列表
     * @return
     */
    public KnowledgeAgentVo info() {
        String userNo = userService.queryLoginUsername();
        Calendar date = Calendar.getInstance();
        date.setTime(new Date());
        LambdaQueryWrapper<KnowledgeAgent> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(KnowledgeAgent::getUserNo,userNo);
        KnowledgeAgent knowledgeAgent = knowledgeAgentMapper.selectOne(wrapper);
        KnowledgeAgentVo knowledgeAgentVo = new KnowledgeAgentVo();
        if (knowledgeAgent == null){
            return knowledgeAgentVo;
        }
        //如果代理时间过期，删除记录，返回null
        Calendar end = Calendar.getInstance();
        end.setTime(knowledgeAgent.getEndTime());
        if (date.after(end)){
            knowledgeAgentMapper.deleteByUserNo(userNo);
            return new KnowledgeAgentVo();
        }
        BeanUtils.copyProperties(knowledgeAgent,knowledgeAgentVo);
        JPaasUser jPaasUser = userService.queryByUsername(knowledgeAgent.getTagerNo());
        knowledgeAgentVo.setTagerName(jPaasUser.getFullName());
        if (knowledgeAgent.getState() == 1){
            knowledgeAgentVo.setStateName("启用");
        } else if (knowledgeAgent.getState() == 2){
            knowledgeAgentVo.setStateName("禁用");
        }
        return knowledgeAgentVo;
    }


    /**
     * 清空代理人
     * @return
     */
    public String del() {
        String userNo = userService.queryLoginUsername();
        knowledgeAgentMapper.del(userNo);
        return "";
    }

    /**
     * 判断是否有代理人且生效中(审核使用)
     * @param knowledgeApprove
     * @return
     */
    public boolean hasTager(KnowledgeApprove knowledgeApprove){
        if (StringUtils.isNotEmpty(knowledgeApprove.getTagerNo())){
            Calendar date = Calendar.getInstance();
            date.setTime(new Date());
            Calendar begin = Calendar.getInstance();
            begin.setTime(knowledgeApprove.getStartTime());
            Calendar end = Calendar.getInstance();
            end.setTime(knowledgeApprove.getEndTime());
            return date.after(begin) && date.before(end);
        } else {
            return false;
        }
    }

    /**
     * 有两个领导，依次查询是否有代理人
     * @param split
     */
    public String hasLeaderMore(String[] split) {
        Date date = new Date();
        for (int i = 0; i < split.length; i++) {
            LambdaQueryWrapper<KnowledgeAgent> wrapper = new LambdaQueryWrapper<>();
            wrapper.eq(KnowledgeAgent::getUserNo,split[i]);
            wrapper.eq(KnowledgeAgent::getState,1);
            KnowledgeAgent knowledgeAgent = knowledgeAgentMapper.selectOne(wrapper);
            if (knowledgeAgent != null){
                if (date.after(knowledgeAgent.getStartTime()) && date.before(knowledgeAgent.getEndTime())){
                    split[i] = knowledgeAgent.getTagerNo();
                }
            }
        }
        String approverId = String.join(",", split);
        return approverId;
    }
}
