
package com.redxun.knowledge.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.model.JPaasUser;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.StringUtils;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.*;
import com.redxun.knowledge.entity.dto.KnowledgeApprovalNodeDto;
import com.redxun.knowledge.entity.dto.KnowledgeApprovalNodeFlowerDto;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalListVo;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalNodeFlowerVo;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalNodeVo;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalVo;
import com.redxun.knowledge.mapper.*;
import lombok.SneakyThrows;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.xml.bind.annotation.XmlType;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * [业务--知识审批节点表]业务服务类
 */
@Service
public class KnowledgeApprovalNodeServiceImpl extends SuperServiceImpl<KnowledgeApprovalNodeMapper, KnowledgeApprovalNode> implements BaseService<KnowledgeApprovalNode> {

    public static final String flowerName = "系统默认流程";

    @Resource
    private KnowledgeApprovalNodeMapper knowledgeApprovalNodeMapper;

    @Autowired
    private KnowledgeApprovalNodeFlowerMapper knowledgeApprovalNodeFlowerMapper;

    @Autowired
    private KnowledgeApprovalNodeCategoryMapper knowledgeApprovalNodeCategoryMapper;

    @Autowired
    private KnowledgeCategoryMapper knowledgeCategoryMapper;

    @Autowired
    private UserService userService;

    @Autowired
    private KnowledgeAgentMapper knowledgeAgentMapper;

    @Autowired
    private KnowledgeAgentServiceImpl knowledgeAgentService;

    @Override
    public BaseDao<KnowledgeApprovalNode> getRepository() {
        return knowledgeApprovalNodeMapper;
    }

    public List<KnowledgeApprovalVo> queryApprovalNodes() {
        return knowledgeApprovalNodeMapper.queryApprovalNodes();
    }

    @Override
    public boolean saveOrUpdateBatch(Collection<KnowledgeApprovalNode> entityList, int batchSize) {
        return super.saveOrUpdateBatch(entityList, batchSize);
    }

    @Override
    public int insert(KnowledgeApprovalNode entity) {
        return 0;
    }

    /**
     * 分页查询审核流程节点
     *
     * @param filter
     * @return
     */
    public IPage findAllApprovalNodes(QueryFilter filter) {
        Map<String, Object> params = PageHelper.constructParams(filter);
        IPage page = knowledgeApprovalNodeMapper.findAllApprovalNodes(filter.getPage(), filter.getParams(), params);
        return page;
    }

    public void deleteKnowledgeApprovalNode() {
        knowledgeApprovalNodeMapper.deleteKnowledgeApprovalNode();
    }

    /**
     * 添加审核流程(新)
     *
     * @param knowledgeApprovalNodeFlowerDto
     */
    @Transactional(rollbackFor = Exception.class)
    public String insertApprovalNodes(KnowledgeApprovalNodeFlowerDto knowledgeApprovalNodeFlowerDto) throws Exception {
        //修改(清除原本数据)
        if (StringUtils.isNotEmpty(knowledgeApprovalNodeFlowerDto.getPkId())) {
            //判断是否能进行修改(创建人或维护人满足一个即可修改)
            KnowledgeApprovalNodeFlower knowledgeApprovalNodeFlower = knowledgeApprovalNodeFlowerMapper.selectById(knowledgeApprovalNodeFlowerDto.getPkId());
            String userId = userService.queryLoginUser();
            if (!userId.equals(knowledgeApprovalNodeFlower.getCreateBy())){
                throw new RuntimeException("您没有权限进行此操作");
            }
            if (StringUtils.isNotEmpty(knowledgeApprovalNodeFlower.getMaintainer())){
                if (!Arrays.asList(knowledgeApprovalNodeFlower.getMaintainer().split(",")).contains(userId)){
                    throw new RuntimeException("您没有权限进行此操作");
                }
            }
            knowledgeApprovalNodeMapper.deleteByFlowerId(knowledgeApprovalNodeFlowerDto.getPkId());
            knowledgeApprovalNodeCategoryMapper.deleteByFlowerId(knowledgeApprovalNodeFlowerDto.getPkId());
        }

        //判断输入的分类是否存在流程
        knowledgeApprovalNodeFlowerDto.getCategoryId().forEach(e -> {
            KnowledgeApprovalNodeCategory knowledgeApprovalNodeCategory = knowledgeApprovalNodeCategoryMapper.
                    selectOne(new LambdaQueryWrapper<KnowledgeApprovalNodeCategory>().eq(KnowledgeApprovalNodeCategory::getCategoryId, e));
            if (!Objects.isNull(knowledgeApprovalNodeCategory)) {
                KnowledgeCategory knowledgeCategory = knowledgeCategoryMapper.selectById(knowledgeApprovalNodeCategory.getCategoryId());
                if (!Objects.isNull(knowledgeCategory)) {
                    throw new RuntimeException("添加失败,关联的分类" + knowledgeCategory.getName() + "已有流程");
                } else {
                    throw new RuntimeException("添加失败,关联的分类不存在");
                }
            }
        });


        //如果名称是默认审核流程,关联分类为默认值
        if (knowledgeApprovalNodeFlowerDto.getFlowerName().equals(flowerName)) {
            Set<String> list = new HashSet<>();
            list.add("-2");
            knowledgeApprovalNodeFlowerDto.setCategoryId(list);
        }
        KnowledgeApprovalNodeFlower knowledgeApprovalNodeFlower = new KnowledgeApprovalNodeFlower();
        knowledgeApprovalNodeFlower.setFlowerName(knowledgeApprovalNodeFlowerDto.getFlowerName());
        knowledgeApprovalNodeFlower.setMaintainer(knowledgeApprovalNodeFlowerDto.getMaintainer());
        if (StringUtils.isEmpty(knowledgeApprovalNodeFlowerDto.getPkId())) {
            knowledgeApprovalNodeFlowerMapper.insert(knowledgeApprovalNodeFlower);
        } else {
            knowledgeApprovalNodeFlower.setPkId(knowledgeApprovalNodeFlowerDto.getPkId());
            knowledgeApprovalNodeFlowerMapper.updateById(knowledgeApprovalNodeFlower);
        }

        //关联分类
        if (!CollectionUtils.isEmpty(knowledgeApprovalNodeFlowerDto.getCategoryId())) {
            knowledgeApprovalNodeFlowerDto.getCategoryId().forEach(e -> {
                KnowledgeApprovalNodeCategory knowledgeApprovalNodeCategory = new KnowledgeApprovalNodeCategory();
                knowledgeApprovalNodeCategory.setCategoryId(e);
                knowledgeApprovalNodeCategory.setFlowerId(knowledgeApprovalNodeFlower.getPkId());
                List<String> pathList = knowledgeCategoryMapper.selectPath(e);
                //knowledgeApprovalNodeCategory.setPath(String.join(".", pathList));
                knowledgeApprovalNodeCategoryMapper.insert(knowledgeApprovalNodeCategory);
            });
        }
        //关联节点
        List<KnowledgeApprovalNodeDto> knowledgeApprovalNodes = knowledgeApprovalNodeFlowerDto.getKnowledgeApprovalNode();
        if (!CollectionUtils.isEmpty(knowledgeApprovalNodes)) {
            AtomicInteger sort = new AtomicInteger(1);
            knowledgeApprovalNodes.forEach(e -> {
                KnowledgeApprovalNode knowledgeApprovalNode = new KnowledgeApprovalNode();
                BeanUtils.copyProperties(e, knowledgeApprovalNode);
                knowledgeApprovalNode.setFlowerId(knowledgeApprovalNodeFlower.getPkId());
                knowledgeApprovalNode.setSort(sort.getAndIncrement());
                knowledgeApprovalNodeMapper.insert(knowledgeApprovalNode);
            });
        }
        return knowledgeApprovalNodeFlower.getPkId();
    }

    /**
     * 查询全部节点
     *
     * @return
     */
    public List<KnowledgeApprovalNodeFlowerVo> findAllNodes() {
        String userId = userService.queryLoginUser();
        List<KnowledgeApprovalNodeFlowerVo> result = knowledgeApprovalNodeFlowerMapper.findAllNodes(userId);
        result.forEach(e -> {
            if (flowerName.equals(e.getFlowerName())) {
                e.setCategory(new ArrayList<>());
            }
            List<String> category = new ArrayList<>();
            e.getCategory().forEach(h -> {
                KnowledgeCategory knowledgeCategory = knowledgeCategoryMapper.selectById(h);
                if (!Objects.isNull(knowledgeCategory)) {
                    category.add(knowledgeCategory.getName());
                }
            });
            e.setCreateByName(userService.queryUser(e.getCreateBy()).getFullName());
            if (StringUtils.isNotEmpty(e.getMaintainer())){
                String maintainerName = Arrays.stream(e.getMaintainer().split(",")).
                        map(maintainer -> userService.queryOsUserDto(maintainer).getFullName()).
                        collect(Collectors.joining(","));
                e.setMaintainerName(maintainerName);
            }
            e.setCategoryName(category);
        });
        return result;
    }

    /**
     * pc端查询审核流程
     *
     * @param categoryId
     * @return
     */
    public List<KnowledgeApprovalNodeVo> findAllNodesByCategoryId(String categoryId,String userId) {
        List<KnowledgeApprovalNodeVo> result = new ArrayList<>();
        //没输入分类id，查看默认流程后返回
        if (StringUtils.isEmpty(categoryId)) {
            categoryId = "-2";
            result = knowledgeApprovalNodeFlowerMapper.findAllNodesByCategoryId(categoryId);
            defaultNode(result.get(0),userId);
            result.forEach(e -> {
                //KnowledgeAgent knowledgeAgent = hasTager(e);
                //if (Objects.nonNull(knowledgeAgent)) {
                //    e.setApproverId(knowledgeAgent.getTagerNo());
                //    e.setApproverName(userService.queryByUsername(knowledgeAgent.getTagerNo()).getFullName());
                //}
                e.setApproverAndActualName(e.getApproverName());
            });
            return result;
        }
        //输入分类id
        result = knowledgeApprovalNodeFlowerMapper.findAllNodesByCategoryId(categoryId);
        //查询其上级是否有审核流程
        if (CollectionUtils.isEmpty(result)) {
            KnowledgeCategory knowledgeCategory = knowledgeCategoryMapper.selectById(categoryId);
            while (true) {
                if ("0".equals(knowledgeCategory.getPkId())) {
                    break;
                }
                //父级有审核模板，退出循环
                knowledgeCategory = knowledgeCategoryMapper.
                        selectOne(new LambdaQueryWrapper<KnowledgeCategory>().eq(KnowledgeCategory::getPkId, knowledgeCategory.getParent()));
                result = knowledgeApprovalNodeFlowerMapper.findAllNodesByCategoryId(knowledgeCategory.getPkId());
                if (!CollectionUtils.isEmpty(result)) {
                    break;
                }
            }
        }
        //该分类没有审核流程,使用默认流程
        if (CollectionUtils.isEmpty(result)) {
            result = knowledgeApprovalNodeFlowerMapper.findAllNodesByCategoryId("-2");
        }
        defaultNode(result.get(0),userId);
        result.forEach(e -> {
            //KnowledgeAgent knowledgeAgent = hasTager(e);
            //if (Objects.nonNull(knowledgeAgent)) {
            //    e.setApproverId(knowledgeAgent.getTagerNo());
            //    e.setApproverName(userService.queryByUsername(knowledgeAgent.getTagerNo()).getFullName());
            //}
            e.setApproverAndActualName(e.getApproverName());
        });
        return result;
    }

    /**
     * 判断是否有代理人且生效中(审核节点详情使用)
     * @param knowledgeApprovalListVo
     * @return
     */
    private KnowledgeAgent hasTager(KnowledgeApprovalNodeVo knowledgeApprovalListVo){
        KnowledgeAgent knowledgeAgent = knowledgeAgentMapper.selectOne(new LambdaQueryWrapper<KnowledgeAgent>().
                eq(KnowledgeAgent::getUserNo, knowledgeApprovalListVo.getApproverId()));
        if (Objects.nonNull(knowledgeAgent)){
            Calendar date = Calendar.getInstance();
            date.setTime(new Date());
            Calendar begin = Calendar.getInstance();
            begin.setTime(knowledgeAgent.getStartTime());
            Calendar end = Calendar.getInstance();
            end.setTime(knowledgeAgent.getEndTime());
            if (date.after(begin) && date.before(end)){
                return knowledgeAgent;
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    /**
     * 处理默认节点
     * @param result
     * @param userId
     */
    private void defaultNode(KnowledgeApprovalNodeVo result,String userId) {
        if (result.getIsFinal() != -1) {
            throw new RuntimeException("没找到默认节点");
        }
        //处理默认节点
        String deptId = null;
        String userNo = userService.queryLoginUsername();
        if (StringUtils.isEmpty(userId)){
            deptId = userService.queryLoginUserDeptId();
        } else {
            deptId = userService.queryOsUserDto(userId).getDeptId();
        }

        String parentUserIds = knowledgeApprovalNodeFlowerMapper.findParentUserIdList(deptId);
        if (!StringUtils.isEmpty(parentUserIds)) {
            List<String> parentUserIdList = Arrays.asList(parentUserIds.split(","));
            //审核人userId(员工工号)
            String approverId = parentUserIdList.parallelStream().map(e -> userService.queryOsUserDto(e).getUserNo()).
                    collect(Collectors.joining(","));

            //如果查出来的直属部门领导包含了当前登陆人，去查询该登陆人的直属上司
            if (approverId.contains(userNo)) {
                approverId = knowledgeApprovalNodeFlowerMapper.findParentUserId(userNo);
                if (StringUtils.isEmpty(approverId)){
                    approverId = userNo;
                }
            }
            //String[] split = approverId.split(",");
            //处理默认节点有两个领导的情况(默认节点默认在第一位)
            //if (split.length > 1){
            //    String approverIdTager = knowledgeAgentService.hasLeaderMore(split);
            //    result.setApproverId(approverIdTager);
            //} else {
            result.setApproverId(approverId);
            //}
            //审核人姓名
            String approverName = Arrays.asList(result.getApproverId().split(",")).parallelStream().map(e -> userService.queryByUsername(e).getFullName()).
                    collect(Collectors.joining(","));
            result.setApproverName(approverName);
        } else {
            String parentUserId;
            if (StringUtils.isEmpty(userId)){
                parentUserId = knowledgeApprovalNodeFlowerMapper.findParentUserId(userService.queryLoginUsername());
                if (StringUtils.isEmpty(parentUserId)){
                    parentUserId = userService.queryLoginUsername();
                }
            } else {
                parentUserId = knowledgeApprovalNodeFlowerMapper.findParentUserId(userService.queryOsUserDto(userId).getUserNo());
                if (StringUtils.isEmpty(parentUserId)){
                    parentUserId = userService.queryOsUserDto(userId).getUserNo();
                }
            }
            result.setApproverId(parentUserId);
            result.setApproverName(userService.queryByUsername(parentUserId).getFullName());
        }
        result.setApproverAndActualName(result.getApproverName());
    }

    /**
     * 删除审核流程
     *
     * @param pkId
     */
    @Transactional(rollbackFor = Exception.class)
    public void delApprovalNode(String pkId) {
        knowledgeApprovalNodeFlowerMapper.deleteByPkId(pkId);
        knowledgeApprovalNodeMapper.deleteByFlowerId(pkId);
        knowledgeApprovalNodeCategoryMapper.deleteByFlowerId(pkId);
    }

    /**
     * 查询选过的关联分类id集
     * @param pkId
     * @return
     */
    public List<String> findAllCategorySelected(String pkId) {
        List<String> idList = knowledgeApprovalNodeCategoryMapper.findAllCategorySelected(pkId);
        List<String> result = idList.stream().filter(e -> !("-2".equals(e))).collect(Collectors.toList());
        return result;
    }
}
