package com.redxun.knowledge.task;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import com.redxun.knowledge.common.MessageService;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalListVo;
import com.redxun.knowledge.mapper.KnowledgeApproveMapper;
import com.redxun.knowledge.qywexin.QYWechatUtils;
import com.redxun.knowledge.service.KnowledgeApproveServiceImpl;
import com.xxl.job.core.handler.IJobHandler;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 文件名: SendQYWXJobHandler
 * 作者: zizg
 * 时间: 2023/7/25
 * 描述: 每天8点给未审核知识超过两天的人发企业微信
 * 修改人:
 * 修改时间:
 * 修改内容:
 */
@Component
@Slf4j
public class SendQYWXJobHandler extends IJobHandler {

    @Autowired
    private KnowledgeApproveServiceImpl knowledgeApproveService;

    @Autowired
    private KnowledgeApproveMapper knowledgeApproveMapper;

    @Autowired
    private UserService userService;

    @Value("${qywx.approvePage}")
    private String qywxApprovePage;

    @Override
    @XxlJob("sendQYWX")
    public void execute() throws Exception {
        log.info("开始发送消息～");
        List<KnowledgeApprovalListVo> allKnowledgeApprove = knowledgeApproveMapper.findAllKnowledgeApprove(new HashMap<>(), new HashMap<>());
        List<KnowledgeApprovalListVo> result = knowledgeApproveService.allApproveList(allKnowledgeApprove);
        //过滤出待审核记录和待最终审核记录
        List<SendMessageDTO> collect = result.parallelStream().filter(e -> e.getApprovalStatus() == 1 || e.getApprovalStatus() == 3).map(e -> {
            SendMessageDTO sendMessageDTO = new SendMessageDTO();
            sendMessageDTO.setApproverId(e.getApproverId());
            sendMessageDTO.setTitles(e.getTitles());
            sendMessageDTO.setUserId(e.getCreateBy());
            sendMessageDTO.setCreateTime(e.getCreateTime());
            return sendMessageDTO;
        }).collect(Collectors.toList());
        log.info("待发送短信人数 -> {}",collect.size());
        collect.forEach(e -> this.sendMessage(e.getApproverId(),e.getTitles(),e.getUserId(),e.getCreateTime()));
        log.info("消息发送完成～");
    }

    /**
     * 发送企业微信
     * @param sender 待发送人
     * @param titles 文章标题
     * @param userId 文章作者
     * @param date   创建日期
     */
    private void sendMessage(String sender, String titles, String userId, Date date){
        String[] qyUser = new  String[]{sender};
        // 发消息通过前来审核
        QYWechatUtils.send(qyUser,"您有一条待审核任务，请尽快完成审核！" + "<br>" +
                "审核类型:知识审核" + "<br>" +
                "提交内容:" + titles + "<br>" +
                "提交人:" + userService.queryUser(userId).getFullName() + "<br>" +
                "提交时间:" + DateUtil.format(date, DatePattern.NORM_DATETIME_PATTERN)+ "<br>" +
                "审批地址:" + "<a href=" + qywxApprovePage + ">查看</a>");
    }
}
